use std::sync::Arc;

use tokio::sync::{Mutex, OnceCell};

use crate::clash_conn_msg::ClashConnectionsWrapper;

#[derive(Clone)]
pub struct StatisticsState {
    pub connections: usize,
    pub download_total: u64,
    pub upload_total: u64,
    pub speed_upload: u64,
    pub speed_download: u64,
}

impl StatisticsState {
    pub fn new() -> Self {
        Self {
            connections: 0,
            download_total: 0,
            upload_total: 0,
            speed_upload: 0,
            speed_download: 0,
        }
    }

    pub fn update(&mut self, wrapper: &ClashConnectionsWrapper) {
        self.connections = wrapper.connections.len();
        self.update_speed(wrapper.upload_total, wrapper.download_total);
        self.download_total = wrapper.download_total;
        self.upload_total = wrapper.upload_total;
    }

    pub fn update_speed(&mut self, curr_upload: u64, curr_download: u64) {
        self.speed_upload = curr_upload - self.upload_total;
        self.speed_download = curr_download - self.download_total;
    }
}

pub struct StatisticsManager {
    state: Arc<Mutex<StatisticsState>>,
}

impl StatisticsManager {
    pub async fn global() -> &'static Self {
        static STATISTICS_MANAGER: OnceCell<StatisticsManager> = OnceCell::const_new();

        STATISTICS_MANAGER
            .get_or_init(|| async { StatisticsManager::new() })
            .await
    }

    pub fn new() -> Self {
        Self {
            state: Arc::new(Mutex::new(StatisticsState::new())),
        }
    }

    pub async fn update(&self, wrapper: &ClashConnectionsWrapper) {
        let mut state = self.state.lock().await;
        state.update(wrapper);
    }

    pub async fn get_state(&self) -> StatisticsState {
        let state = self.state.lock().await;
        state.clone()
    }
}
